'use strict';

var p = require('path');
// const printAST = require('ast-pretty-print')

module.exports = macrosPlugin;

function macrosPlugin() {
  return {
    name: 'macros',
    visitor: {
      ImportDeclaration(path, state) {
        var isMacros = looksLike(path, {
          node: {
            source: {
              value: function value(v) {
                return v.endsWith('.macros');
              }
            }
          }
        });
        if (!isMacros) {
          return;
        }
        var name = path.node.specifiers[0].local.name;
        var source = path.node.source.value;
        applyMacros({ path, name, source, state });
        path.remove();
      },
      CallExpression(path, state) {
        var isMacros = looksLike(path, {
          node: {
            callee: {
              type: 'Identifier',
              name: 'require'
            },
            arguments: function _arguments(args) {
              return args.length === 1 && args[0].value.endsWith('.macros');
            }
          },
          parent: {
            type: 'VariableDeclarator'
          }
        });
        if (!isMacros) {
          return;
        }
        var name = path.parent.id.name;
        var source = path.node.arguments[0].value;
        applyMacros({ path, name, source, state });
        path.parentPath.remove();
      }
    }
  };
}

function applyMacros(_ref) {
  var path = _ref.path,
      name = _ref.name,
      source = _ref.source,
      state = _ref.state;
  var filename = state.file.opts.filename;

  var referencePaths = path.scope.getBinding(name).referencePaths;
  if (referencePaths && referencePaths.length) {
    var requirePath = p.join(p.dirname(filename), source);
    // eslint-disable-next-line import/no-dynamic-require
    var macros = require(requirePath);
    referencePaths.forEach(function (ref) {
      // TODO: if the macros doesn't support one of the methods,
      // and someone tries to use it that way, then throw a helpful
      // error message
      if (ref.parentPath.type === 'TaggedTemplateExpression') {
        macros.asTag(ref.parentPath.get('quasi'), state);
      } else if (ref.parentPath.type === 'CallExpression') {
        macros.asFunction(ref.parentPath.get('arguments'), state);
      } else if (ref.parentPath.type === 'JSXOpeningElement') {
        macros.asJSX({
          attributes: ref.parentPath.get('attributes'),
          children: ref.parentPath.parentPath.get('children')
        }, state);
      } else {
        // TODO: throw a helpful error message
      }
    });
  }
}

function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}